#include <stdio.h>
#include <string.h>

const long MAX_FIELD_SIZE_X = 8; //8
const long MAX_FIELD_SIZE_Y = 8; //8
const long MAX_PATH_LENGTH = 10; //8
const char START_LETTER = '@';
const long LETTERS_PAIRS_COUNT = 7; //7
const char LETTERS_PAIRS[7][2] =
{
	{'@', 'I'},
	{'I', 'E'},
	{'E', 'H'},
	{'H', 'O'},
	{'O', 'V'},
	{'V', 'A'},
	{'A', '#'}
};

template<class T>
struct tVector2D
{
	tVector2D(T initX=0, T initY=0) : x(initX), y(initY) {}
	
	tVector2D operator-(const tVector2D& toSub)
	{
		return tVector2D(x-toSub.x, y-toSub.y);
	}
	tVector2D operator+(const tVector2D& toAdd)
	{
		return tVector2D(x+toAdd.x, y+toAdd.y);
	}
	T operator%(const tVector2D& toMult)
	{
		return x*toMult.y - y*toMult.x;
	}
	T operator*(const tVector2D& toMult)
	{
		return x*toMult.x + y*toMult.y;
	}
	double Length()
	{
		return sqrt(x*x+y*y);
	}
	int operator==(const tVector2D &vector)
	{
		return x == vector.x && y == vector.y;
	}

	inline void Read(void);

	T x, y;
};

typedef tVector2D<long> tVector2DLong;

const long DIRECTIONS_COUNT = 3; //3
const tVector2DLong DIRECTION_VECTORS[DIRECTIONS_COUNT] =
{
	tVector2DLong(0, -1),
	tVector2DLong(-1, 0),
	tVector2DLong(1, 0)
};

const char * const DIRECTION_NAMES[DIRECTIONS_COUNT] =
{
	"forth",
	"left",
	"right"
};


template<class T, long TSize>
struct tQueue
{
	T data[TSize];
	long head, tail;

	tQueue(void)
	{
		Reset();
	}

	void Reset(void)
	{
		head = tail = 0;
	}

	void Put(const T &value)
	{
		data[tail++] = value;
	}

	const T& Get(void)
	{
		return data[head++];
	}

	int Empty(void)
	{
		return head == tail;
	}
};

struct tField
{
	char field[MAX_FIELD_SIZE_Y][MAX_FIELD_SIZE_X];
	tVector2DLong fieldSize;

	void Read(void)
	{
		scanf("%ld %ld", &fieldSize.y, &fieldSize.x);
		char buffer[MAX_FIELD_SIZE_X + 1];
		long x,y;
		for(y = 0; y < fieldSize.y; ++y)
		{
			scanf("%s", buffer);
			for(x = 0; x < fieldSize.x; ++x)
				field[y][x] = buffer[x];
		}
	}

	tVector2DLong GetStartPoint(void)const
	{
		long x, y;
		for(y = 0; y < fieldSize.y; ++y)
			for(x = 0; x < fieldSize.x; ++x)
				if(field[y][x] == START_LETTER)
					return tVector2DLong(x, y);
		return tVector2DLong(0, 0);
	}

	int PointOnField(tVector2DLong point)const
	{
		return point.x >= 0 && point.y >= 0 && point.x < fieldSize.x && point.y < fieldSize.y;
	}

	char GetLetter(tVector2DLong point)const
	{
		if(PointOnField(point))return field[point.y][point.x];
		else return ' ';
	}

	int CanGo(tVector2DLong startPoint, tVector2DLong endPoint)const
	{
		char startLetter = GetLetter(startPoint);
		char endLetter = GetLetter(endPoint);
		long iPair;
		for(iPair = 0; iPair < LETTERS_PAIRS_COUNT; ++iPair)
			if(LETTERS_PAIRS[iPair][0] == startLetter && LETTERS_PAIRS[iPair][1] == endLetter)
				return 1;
		return 0;
	}

	void WriteDirection(tVector2DLong startPoint, tVector2DLong endPoint)const
	{
		tVector2DLong dirVec = endPoint - startPoint;
		long iDir;
		for(iDir = 0; iDir < DIRECTIONS_COUNT; ++iDir)
			if(dirVec == DIRECTION_VECTORS[iDir])
				printf("%s", DIRECTION_NAMES[iDir]);
	}

	void BuildAndWritePath(void)const
	{
		tQueue<tVector2DLong, MAX_PATH_LENGTH> queue;

		queue.Put(GetStartPoint());
		int firstMove = 1;
		while(!queue.Empty())
		{
			tVector2DLong currentPoint = queue.Get();
			long iDir;
			for(iDir = 0; iDir < DIRECTIONS_COUNT; ++iDir)
			{
				tVector2DLong nextPoint = currentPoint + DIRECTION_VECTORS[iDir];
				if(CanGo(currentPoint, nextPoint))
				{
					if(firstMove)firstMove = 0;
					else putchar(' ');
					WriteDirection(currentPoint, nextPoint);
					queue.Put(nextPoint);
				}
			}
		}
		putchar('\n');
	}
};

int main()
{
#ifndef ONLINE_JUDGE
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
#endif

	long cTests, iTest;
	scanf("%ld", &cTests);
	for(iTest = 0; iTest < cTests; ++iTest)
	{
		tField field;
		field.Read();
		field.BuildAndWritePath();
	}

#ifndef ONLINE_JUDGE
	fclose(stdout);
	fclose(stdin);
#endif
	return 0;
}
